package info.textgrid.middleware.test.online.tg;

import static org.junit.Assert.assertTrue;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import javax.ws.rs.core.Response;
import org.apache.cxf.helpers.IOUtils;
import org.apache.cxf.jaxrs.client.Client;
import org.apache.http.HttpStatus;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.test.online.OAIPMHUtilitiesOnline;

/**
 * <p>
 * Some online tests for the TextGrid OAIMPH service --> verb=GetRecord <--
 * </p>
 * 
 * <p>
 * Please set PROPERTIES_FILE and TEST_ALL_PAGES in class <b>OAIPMHUtilitiesOnline</b>!
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-23
 * @since 2022-09-08
 */
//@Ignore
public class TestTGGetRecordOnline {

  // **
  // STATICS
  // **

  // Some JAXRS things.
  private static String oaipmhEndpoint;
  static Client oaipmhWebClient;

  // Properties
  private static String checkGetRecordDC;
  private static String expectedGetRecordDC;

  private static String checkGetRecordDATACITE;
  private static String expectedGetRecordDATACITE;

  private static String checkGetRecordIDIOM;
  private static String expectedGetRecordIDIOM;

  private static String checkGetRecordIDIOMImage;
  private static String expectedGetRecordIDIOMImage;

  private static List<String> checkGetRecordIDList = new ArrayList<String>();

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Get properties.
    Properties p =
        OAIPMHUtilitiesOnline.getPropertiesFromFile(OAIPMHUtilitiesOnline.PROPERTIES_FILE);

    // Set properties.
    oaipmhEndpoint = p.getProperty("oaipmhEndpoint");

    checkGetRecordDC = p.getProperty("checkGetRecordDC");
    expectedGetRecordDC = p.getProperty("expectedGetRecordDC");

    checkGetRecordDATACITE = p.getProperty("checkGetRecordDATACITE");
    expectedGetRecordDATACITE = p.getProperty("expectedGetRecordDATACITE");

    checkGetRecordIDIOM = p.getProperty("checkGetRecordIDIOM");
    expectedGetRecordIDIOM = p.getProperty("expectedGetRecordIDIOM");

    checkGetRecordIDIOMImage = p.getProperty("checkGetRecordIDIOMImage");
    expectedGetRecordIDIOMImage = p.getProperty("expectedGetRecordIDIOMImage");

    checkGetRecordIDList =
        OAIPMHUtilitiesOnline.getListFromProperties((String) p.get("checkGetRecordIDList"));

    // Get web client from endpoint.
    oaipmhWebClient = OAIPMHUtilitiesOnline.getOAIPMHWEebClient(oaipmhEndpoint);
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {
    //
  }

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordOAIDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testGetRecordOAIDC()");

    String query = "verb=" + OAIPMHUtilitiesOnline.VERB_GET_RECORD + "&identifier="
        + checkGetRecordDC + "&metadataPrefix=" + OAIPMHUtilitiesOnline.OAI_DC_PREFIX;

    Response httpResponse = OAIPMHUtilitiesOnline.getOAIHttpResponse(oaipmhWebClient, query);
    int status = httpResponse.getStatus();

    String response = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());

    if (status != HttpStatus.SC_OK || !response.contains(expectedGetRecordDC)) {
      String message = "[" + status + "] response should contain '" + expectedGetRecordDC + "'";
      assertTrue(message, false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordIDIOMMETS() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testGetRecordIDIOMMETS()");

    String query = "verb=" + OAIPMHUtilitiesOnline.VERB_GET_RECORD + "&identifier="
        + checkGetRecordIDIOM + "&metadataPrefix=" + OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX;

    Response httpResponse = OAIPMHUtilitiesOnline.getOAIHttpResponse(oaipmhWebClient,
        query);
    int status = httpResponse.getStatus();

    String response = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());

    if (status != HttpStatus.SC_OK || !response.contains(expectedGetRecordIDIOM)) {
      String message = "[" + status + "] response should contain '" + expectedGetRecordIDIOM + "'";
      assertTrue(message, false);
    }

    // Test OAI header.
    OAIPMHUtilitiesOnline.examineTGHeaderIDs(response, OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX);

    System.out.println("\tresponse: " + response);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordIDIOMMETSImage() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testGetRecordIDIOMMETS_Image()");

    String query =
        "verb=" + OAIPMHUtilitiesOnline.VERB_GET_RECORD + "&identifier=" + checkGetRecordIDIOMImage
            + "&metadataPrefix=" + OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX;

    Response httpResponse = OAIPMHUtilitiesOnline.getOAIHttpResponse(oaipmhWebClient,
        query);
    int status = httpResponse.getStatus();

    String response = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());

    if (status != HttpStatus.SC_OK || !response.contains(expectedGetRecordIDIOMImage)) {
      String message =
          "[" + status + "] response should contain '" + expectedGetRecordIDIOMImage + "'";
      assertTrue(message, false);
    }

    // Test OAI header.
    OAIPMHUtilitiesOnline.examineTGHeaderIDs(response, OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX);

    System.out.println("\tresponse: " + response);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordDATACITE() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testGetRecordDATACITE()");

    String query = "verb=" + OAIPMHUtilitiesOnline.VERB_GET_RECORD + "&identifier="
        + checkGetRecordDATACITE + "&metadataPrefix=" + OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX;

    Response httpResponse = OAIPMHUtilitiesOnline.getOAIHttpResponse(oaipmhWebClient,
        query);
    int status = httpResponse.getStatus();

    String response = IOUtils.readStringFromStream((InputStream) httpResponse.getEntity());

    if (status != HttpStatus.SC_OK || !response.contains(expectedGetRecordDATACITE)) {
      String message =
          "[" + status + "] response should contain '" + expectedGetRecordDATACITE + "'";
      assertTrue(message, false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testConcurrentlyOAIDCGetRecord() throws InterruptedException, ExecutionException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testConcurrentlyOAIDCGetRecord()");

    ExecutorService executor = Executors.newFixedThreadPool(18);

    Future<Boolean> f1 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(0), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f2 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(1), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f3 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(2), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f4 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(3), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f5 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(4), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f6 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(5), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f7 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(6), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f8 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(7), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f9 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(8), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f10 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(9), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f11 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(10), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f12 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(11), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f13 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(12), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f14 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(13), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f15 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(14), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f16 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(15), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f17 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(16), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));
    Future<Boolean> f18 =
        executor.submit(new OaiPmhGetRecordTestThread(OAIPMHUtilitiesOnline.VERB_GET_RECORD,
            checkGetRecordIDList.get(17), OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            "GR[" + checkGetRecordIDList.get(0) + "]"));

    executor.shutdown();

    System.out.println(OAIPMHUtilitiesOnline.OK + "[GR1]=" + f1.get() +
        ", [GR2]=" + f2.get() +
        ", [GR3]=" + f3.get() +
        ", [GR4]=" + f4.get() +
        ", [GR5]=" + f5.get() +
        ", [GR6]=" + f6.get() +
        ", [GR7]=" + f7.get() +
        ", [GR8]=" + f8.get() +
        ", [GR9]=" + f9.get() +
        ", [GR10]=" + f10.get() +
        ", [GR11]=" + f11.get() +
        ", [GR12]=" + f12.get() +
        ", [GR13]=" + f13.get() +
        ", [GR14]=" + f14.get() +
        ", [GR15]=" + f15.get() +
        ", [GR16]=" + f16.get() +
        ", [GR17]=" + f17.get() +
        ", [GR18]=" + f18.get());
  }

  /**
   * <p>
   * Starts a new OAI-PMH thread.
   * </p>
   */
  private class OaiPmhGetRecordTestThread implements Callable<Boolean> {

    protected String verb;
    protected String identifier;
    protected String metadataPrefix;
    protected String threadName;

    /**
     * @param theVerb
     * @param theIdentifier
     * @param theMetadataPrefix
     * @param theThreadName
     */
    public OaiPmhGetRecordTestThread(String theVerb, String theIdentifier, String theMetadataPrefix,
        String theThreadName) {
      this.verb = theVerb;
      this.identifier = theIdentifier;
      this.metadataPrefix = theMetadataPrefix;
      this.threadName = theThreadName;
    }

    /**
     *
     */
    @Override
    public Boolean call() throws IOException {

      String query = "verb=" + this.verb + "&identifier=" + this.identifier + "&metadataPrefix="
          + this.metadataPrefix;
      OAIPMHUtilitiesOnline.getOAIHttpResponse(oaipmhWebClient, this.threadName, query);

      return true;
    }
  }

}
